/**
  **************************************************************************
  * @file     at32_video_ev_bf3901.c
  * @version  v2.0.2
  * @date     2022-09-15
  * @brief    at32_video_ev_bf3901 program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32_video_ev_bf3901.h"
#include "at32_video_ev_bf3901cfg.h"
#include "at32_video_ev_sccb.h"
#include "at32_video_ev_exint.h"

u8 tempXXY;

/**
  * @brief  initialize bf3901 power down pin
  * @param  none
  * @retval none
  */
void bf3901_pwdn_pin_init(void)
{
  gpio_init_type  GPIO_InitStructure;
  crm_periph_clock_enable(BF3901_PWDN_GPIO_CLK, TRUE);
  /* power down */
  gpio_default_para_init(&GPIO_InitStructure);
  GPIO_InitStructure.gpio_pins = BF3901_PWDN_PIN;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_OUTPUT;
  gpio_init(BF3901_PWDN_GPIO, &GPIO_InitStructure);
  gpio_bits_reset(BF3901_PWDN_GPIO, BF3901_PWDN_PIN);
}

/**
  * @brief  bf3901 power down pin operation
  * @param  none
  * @retval none
  */
void bf3901_pwdn_pin_op(void)
{
  delay_us(50);
  gpio_bits_set(BF3901_PWDN_GPIO, BF3901_PWDN_PIN);
  delay_us(50);
  gpio_bits_reset(BF3901_PWDN_GPIO, BF3901_PWDN_PIN);
  delay_us(100);
  gpio_bits_reset(BF3901_PWDN_GPIO, BF3901_PWDN_PIN);
}

/**
  * @brief  bf3901 power down pin operation1
  * @param  none
  * @retval none
  */
void bf3901_pwdn_pin_op1(void)
{
  gpio_bits_set(BF3901_PWDN_GPIO, BF3901_PWDN_PIN);
  delay_us(50);
  gpio_bits_reset(BF3901_PWDN_GPIO, BF3901_PWDN_PIN);
  delay_us(50);
  gpio_bits_set(BF3901_PWDN_GPIO, BF3901_PWDN_PIN);
  delay_us(100);
  gpio_bits_reset(BF3901_PWDN_GPIO, BF3901_PWDN_PIN);
}

/**
  * @brief  initialize bf3901
  * @param  none
  * @retval 0: success, other: error
  */
u8 bf3901_init(void)
{
  u16 i = 0;

  tempXXY = sccb_rd_reg(0x1C);

  if( tempXXY == 0x7F)
  {
    tempXXY = sccb_rd_reg(0x1D);

    if( tempXXY != 0xA2)
    {
      //tmpCntXXY++;
      return 2;
    }
  }
  else
  {
    return 2;
  }

  /* initialize sequence */
  for(i = 0; i < sizeof(BF3901_RGB_240X320) / sizeof(BF3901_RGB_240X320[0]); i++)
  {
    sccb_wr_reg(BF3901_RGB_240X320[i][0], BF3901_RGB_240X320[i][1]);
  }

  return 0x00; 	//ok
}

/**
  * @brief  configure bf3901 light mode
  * @param  mode
  *         0: auto
  *         1: sunny
  *         2: cloudy
  *         3: office
  *         4: home
  * @retval none
  */
void bf3901_light_mode(u8 mode)
{
  /* default light mode auto */
  u8 regbdval = 0X70;
  u8 regbeval = 0X58;
  u8 regbfval = 0X80;

  switch(mode)
  {
  case 1://sunny
    regbdval = 0XE5;
    regbeval = 0X5A;
    regbfval = 0X5C;
    break;

  case 2://cloudy
    regbdval = 0XE5;
    regbeval = 0X58;
    regbfval = 0X60;
    break;

  case 3://office
    regbdval = 0XE5;
    regbeval = 0X84;
    regbfval = 0X4c;
    break;

  case 4://home
    regbdval = 0XE5;
    regbeval = 0X96;
    regbfval = 0X40;
    break;
  }

  sccb_wr_reg(0Xbd, regbdval); //set com8
  sccb_wr_reg(0Xbe, regbeval); //set AWB blue
  sccb_wr_reg(0Xbf, regbfval); //set AWB red
}

/**
  * @brief  configure bf3901 color saturation
  * @param  sat
  *         0: -2
  *         1: -1
  *         2: 0
  *         3: 1
  *         4: 2
  * @retval none
  */
void bf3901_color_saturation(u8 sat)
{
  /* default color saturation 2 */
  u8 reg80val = 0X40;
  u8 reg81val = 0X38;
  u8 reg82val = 0X38;

  switch(sat)
  {
  case 0://-2
    reg80val = 0X40;
    reg81val = 0X11;
    reg82val = 0X2F;
    break;

  case 1://-1
    reg80val = 0X66;
    reg81val = 0X1B;
    reg82val = 0X4B;
    break;

  case 3://1
    reg80val = 0X99;
    reg81val = 0X28;
    reg82val = 0X71;
    break;

  case 4://2
    reg80val = 0XC0;
    reg81val = 0X33;
    reg82val = 0X8D;
    break;
  }

  sccb_wr_reg(0X80, reg80val);
  sccb_wr_reg(0X81, reg81val);
  sccb_wr_reg(0X82, reg82val);
}

/**
  * @brief  configure bf3901 brightness
  * @param  bright
  *         0: -2
  *         1: -1
  *         2: 0
  *         3: 1
  *         4: 2
  * @retval none
  */
void bf3901_brightness(u8 bright)
{
  /* default brightness 2 */
  u8 reg89val = 0Xe3;

  switch(bright)
  {
  case 0://-2
    reg89val = 0XB0;
    break;

  case 1://-1
    reg89val = 0X98;
    break;

  case 3://1
    reg89val = 0X18;
    break;

  case 4://2
    reg89val = 0X30;
    break;
  }

  sccb_wr_reg(0X89, reg89val);
}

/**
  * @brief  configure bf3901 contrast
  * @param  bright
  *         0: -2
  *         1: -1
  *         2: 0
  *         3: 1
  *         4: 2
  * @retval none
  */
void bf3901_contrast(u8 contrast)
{
  /* default contrast 2 */
  u8 reg84val = 0X90;

  switch(contrast)
  {
  case 0://-2
    reg84val = 0X30;
    break;

  case 1://-1
    reg84val = 0X38;
    break;

  case 3://1
    reg84val = 0X50;
    break;

  case 4://2
    reg84val = 0X60;
    break;
  }

  sccb_wr_reg(0X84, reg84val);
}

/**
  * @brief  configure bf3901 special effects
  * @param  bright
  *         0: normal
  *         1: negative film
  *         2: black-white
  *         3: reddish
  *         4: greenish
  *         5: bluish
  *         6: retro
  * @retval none
  */
void bf3901_special_effects(u8 eft)
{
  /* default special effects normal */
  u8 reg3aval = 0X04;
  u8 reg67val = 0XC0;
  u8 reg68val = 0X80;

  switch(eft)
  {
  case 1://negative film
    reg3aval = 0X24;
    reg67val = 0X80;
    reg68val = 0X80;
    break;

  case 2://black-white
    reg3aval = 0X14;
    reg67val = 0X80;
    reg68val = 0X80;
    break;

  case 3://reddish
    reg3aval = 0X14;
    reg67val = 0Xc0;
    reg68val = 0X80;
    break;

  case 4://greenish
    reg3aval = 0X14;
    reg67val = 0X40;
    reg68val = 0X40;
    break;

  case 5://bluish
    reg3aval = 0X14;
    reg67val = 0X80;
    reg68val = 0XC0;
    break;

  case 6://retro
    reg3aval = 0X14;
    reg67val = 0XA0;
    reg68val = 0X40;
    break;
  }

  sccb_wr_reg(0X3A, reg3aval);
  sccb_wr_reg(0X68, reg67val);
  sccb_wr_reg(0X67, reg68val);
}

/**
  * @brief  set window
  * @param  sx: start x
  * @param  sy: start y
  * @param  width: width
  * @param  height: height
  * @retval none
  */
void bf3901_window_set(u16 sx, u16 sy, u16 width, u16 height)
{
  sccb_wr_reg(0X39, sy);
  sccb_wr_reg(0X3b, height >> 8);
  sccb_wr_reg(0X3c, height);

  sccb_wr_reg(0X3a, sx);
  sccb_wr_reg(0X3e, width);
}
