/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.2
  * @date     2022-09-15
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "string.h"
#include "stdio.h"
#include "at32_video_ev_lcd.h"
#include "at32_video_ev_bf3901.h"
#include "at32_video_ev_exint.h"
#include "at32_video_ev_spi.h"
#include "at32_video_ev_malloc.h"
#include "Artery_QR.h"
#include "usbd_core.h"
#include "cdc_class.h"
#include "cdc_desc.h"
#include "usbd_int.h"
#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"

typedef enum {failed = 0, passed = !failed} teststatus;
extern volatile u16 RxBuffBankIndx;
extern u8  DMA_Rx_Buffer[2][DMARxSize];
#if(LCD_SPI_TX_DMA_EN)
extern u8  LcdSpiDMA_Tx_Buffer[LCDSPIDMATxSize];
#endif

#define Only_detect_QR //only decode QR Code
//#define Detect_all   //decode bar code and QR code

int Key_run = 0;
u8 bmp[57600] = {0};

__IO u8 FrameCnt = 0;
__IO u8 FrameFre = 0;
__IO u8  Data_ID = 0;
__IO u16  RxIdx = 0;
__IO u16  Image_Width = 0;
__IO u16  Image_Height = 0;
__IO u16  Line_Number = 0;
__IO u16  Data_Packet_SizeBk = 0;
__IO u8  Frame_Refresh_Flg = 0;
__IO u8  Line_Refresh_Flg = 0;
__IO u8  Fr_Status = 0;
__IO u8  Ln_Status = 0;
__IO u8  FrameFreq_Refresh_Flg = 0;
int Get_Frame = 0;
u16 Hv_sum = 0;
u16 recvLen, sendLen;
int Line_number = 0;
usbd_core_type usb_core_dev;

/**
  * @brief  usb 48M clock select
  * @param  clk_s:USB_CLK_HICK, USB_CLK_HEXT
  * @retval none
  */
void usb_clock48m_select(usb_clk48_s clk_s)
{
  if(clk_s == USB_CLK_HICK)
  {
    crm_usb_clock_source_select(CRM_USB_CLOCK_SOURCE_HICK);

    /* enable the acc calibration ready interrupt */
    crm_periph_clock_enable(CRM_ACC_PERIPH_CLOCK, TRUE);

    /* update the c1\c2\c3 value */
    acc_write_c1(7980);
    acc_write_c2(8000);
    acc_write_c3(8020);

    /* open acc calibration */
    acc_calibration_mode_enable(ACC_CAL_HICKTRIM, TRUE);
  }
  else
  {
    switch(system_core_clock)
    {
      /* 48MHz */
      case 48000000:
        crm_usb_clock_div_set(CRM_USB_DIV_1);
        break;

      /* 72MHz */
      case 72000000:
        crm_usb_clock_div_set(CRM_USB_DIV_1_5);
        break;

      /* 96MHz */
      case 96000000:
        crm_usb_clock_div_set(CRM_USB_DIV_2);
        break;

      /* 120MHz */
      case 120000000:
        crm_usb_clock_div_set(CRM_USB_DIV_2_5);
        break;

      /* 144MHz */
      case 144000000:
        crm_usb_clock_div_set(CRM_USB_DIV_3);
        break;

      /* 168MHz */
      case 168000000:
        crm_usb_clock_div_set(CRM_USB_DIV_3_5);
        break;

      /* 192MHz */
      case 192000000:
        crm_usb_clock_div_set(CRM_USB_DIV_4);
        break;

      default:
        break;

    }
  }
}

/**
  * @brief  camera refresh
  * @param  none
  * @retval none
  */
void camera_refresh(void)
{
  int j;
  u16 Color;

  if((Frame_Refresh_Flg  == FRAME_START_SYNC_FLG) && (Line_Refresh_Flg == LINE_START_FLG ))
  {
    Frame_Refresh_Flg  = FRAME_START_FLG;
  }

  if(Line_Refresh_Flg == LINE_START_FLG)
  {
    if(Key_run != 0x66)
    {
      LCD_RS_SET;
      LCD_SPI_MASTER_Tx_DMA_Channel->ctrl_bit.chen = FALSE;
      LCD_SPI_MASTER_Tx_DMA_Channel->maddr = (uint32_t)( (uint32_t)(DMA_Rx_Buffer + (RxBuffBankIndx ^ 1)) + HsyncSize);
      LCD_SPI_MASTER_Tx_DMA_Channel->dtcnt = LCDSPIDMATxSize;

      LCD_SPI_MASTER_Tx_DMA_Channel->ctrl_bit.chen = TRUE;
    }

    Line_number ++;
    Line_Refresh_Flg = RX_IDLE_FLG;
    Data_Packet_SizeBk = 0;

    Hv_sum++;

    if(Hv_sum  >= IMAGE_HEIGHT)
    {
      /* last line */
      for(j = 0; j < 240; j++)
      {
        /* convert RGB565 into grayscale */
        Color = (DMA_Rx_Buffer[RxBuffBankIndx ^ 1][6 + j * 2] << 8) | (DMA_Rx_Buffer[RxBuffBankIndx ^ 1][6 + j * 2 + 1]);
        bmp[(Hv_sum - 1) * 240 + j] = (((Color & 0xF800) >> 8) * 76 + ((Color & 0x7E0) >> 3) * 150 + ((Color & 0x001F) << 3) * 30) >> 8;
      }

      Get_Frame = 1;
      BF3901_SPI_SLAVE_Rx_DMA_Channel->ctrl_bit.chen = FALSE;
      Hv_sum = 0;
      Frame_Refresh_Flg  = 0;
    }
    else
    {

      BF3901_SPI_SLAVE_Rx_DMA_Channel->ctrl_bit.chen = FALSE;
      BF3901_SPI_SLAVE_Rx_DMA_Channel->maddr =  (uint32_t)(DMA_Rx_Buffer + RxBuffBankIndx);
      BF3901_SPI_SLAVE_Rx_DMA_Channel->dtcnt = DMARxSize;
      BF3901_SPI_SLAVE_Rx_DMA_Channel->ctrl_bit.chen = TRUE;

      for(j = 0; j < 240; j++)
      {
        /* convert RGB565 into grayscale */
        Color = (DMA_Rx_Buffer[RxBuffBankIndx ^ 1][6 + j * 2] << 8) | (DMA_Rx_Buffer[RxBuffBankIndx ^ 1][6 + j * 2 + 1]);
        bmp[(Hv_sum - 1) * 240 + j] = (((Color & 0xF800) >> 8) * 76 + ((Color & 0x7E0) >> 3) * 150 + ((Color & 0x001F) << 3) * 30) >> 8;
      }

      RxBuffBankIndx ^= 1;
    }
  }
}

/**
  * @brief  initializ lcd display information
  * @param  none
  * @retval none
  */
void lcd_display_initial_info(void)
{
  lcd_scan_dir(DFT_SCAN_DIR);
  POINT_COLOR = RED;
  lcd_clear(WHITE);
  lcd_showstring(5, 10, 200, 16, 16, "AT32F403A+BF3901");
  lcd_showstring(5, 30, 250, 16, 16, "@Arterytek, 2018/5/04");
  lcd_showstring(5, 50, 200, 16, 16, "BF3901 Init...");
}

/**
  * @brief  initializ external interrupt line 12
  * @param  none
  * @retval none
  */
void exint12_init(void)
{
  exint_init_type EXINT_InitStructure;
  gpio_init_type	 GPIO_InitStructure;

  crm_periph_clock_enable(CRM_GPIOB_PERIPH_CLOCK, TRUE);

  gpio_default_para_init(&GPIO_InitStructure);
  GPIO_InitStructure.gpio_pins = GPIO_PINS_12;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_INPUT;
  GPIO_InitStructure.gpio_pull = GPIO_PULL_UP;
  gpio_init(GPIOB, &GPIO_InitStructure);

  gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOB, (GPIO_PINS_SOURCE12));

  EXINT_InitStructure.line_select = EXINT_LINE_12;
  EXINT_InitStructure.line_mode = EXINT_LINE_INTERRUPUT;
  EXINT_InitStructure.line_polarity = EXINT_TRIGGER_RISING_EDGE;
  EXINT_InitStructure.line_enable = TRUE;
  exint_init(&EXINT_InitStructure);
}

/**
  * @brief  initializ external interrupt line 14
  * @param  none
  * @retval none
  */
void exint14_init(void)
{
  exint_init_type EXINT_InitStructure;
  gpio_init_type	 GPIO_InitStructure;

  crm_periph_clock_enable(CRM_GPIOB_PERIPH_CLOCK, TRUE);

  gpio_default_para_init(&GPIO_InitStructure);
  GPIO_InitStructure.gpio_pins = GPIO_PINS_14;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_INPUT;
  GPIO_InitStructure.gpio_pull = GPIO_PULL_UP;
  gpio_init(GPIOB, &GPIO_InitStructure);

  gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOB, (GPIO_PINS_SOURCE14));

  EXINT_InitStructure.line_select = EXINT_LINE_14;
  EXINT_InitStructure.line_mode = EXINT_LINE_INTERRUPUT;
  EXINT_InitStructure.line_polarity = EXINT_TRIGGER_RISING_EDGE;
  EXINT_InitStructure.line_enable = TRUE;
  exint_init(&EXINT_InitStructure);
}

/**
  * @brief  nvic configuration
  * @param  none
  * @retval none
  */
void nvic_configuration(void)
{
  /* 2 bit for pre-emption priority,2 bits for subpriority */
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);

  /* configure dma1 channel4 for the spi recevie */
  nvic_irq_enable(BF3901_SPI_SLAVE_Rx_DMA_IRQn, 0, 1);

  /* configure dma1 channel3 for the spi transmit */
  nvic_irq_enable(LCD_SPI_MASTER_Tx_DMA_IRQn, 0, 2);

  /* configure usart1 nvic */
  nvic_irq_enable(USART1_IRQn, 3, 0);

  /* configure tmr6 nvic */
  nvic_irq_enable(TMR6_GLOBAL_IRQn, 3, 1);

  /* configure exint15_10 nvic */
  nvic_irq_enable(EXINT15_10_IRQn, 0, 0);

}

/**
  * @brief  enable spi dma
  * @param  none
  * @retval none
  */
void dma_spix_enable(void)
{
  /* transfer complete */
  while (spi_i2s_flag_get(LCD_SPI_SELECTED, SPI_I2S_BF_FLAG) );

  spi_enable(LCD_SPI_SELECTED, FALSE);

  /* enable lcd_spi_selected dma tx request */
  spi_i2s_dma_transmitter_enable(LCD_SPI_SELECTED, TRUE);

  /* enable lcd_spi_selected */
  spi_enable(LCD_SPI_SELECTED, TRUE);

  dma_interrupt_enable(LCD_SPI_MASTER_Tx_DMA_Channel, DMA_FDT_INT, TRUE);
}

/**
  * @brief  clock output configuration
  * @param  none
  * @retval none
  */
void clock_output_config(void)
{
  gpio_init_type  GPIO_InitStructure;

  crm_periph_clock_enable(CRM_GPIOA_PERIPH_CLOCK, TRUE);
  crm_periph_clock_enable(CRM_IOMUX_PERIPH_CLOCK, TRUE);

  gpio_default_para_init(&GPIO_InitStructure);
  GPIO_InitStructure.gpio_pins = GPIO_PINS_8;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_MUX;
  GPIO_InitStructure.gpio_drive_strength =  GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init(GPIOA, &GPIO_InitStructure);

  /* adcclk = 12.5m */
  crm_adc_clock_div_set(CRM_ADC_DIV_6);

  crm_clock_out_set(CRM_CLKOUT_ADC);
  crm_clkout_div_set(CRM_CLKOUT_DIV_2);
}

/**
  * @brief  compares two buffers.
  * @param  pbuffer1, pbuffer2: buffers to be compared.
  * @param  bufferlength: buffer's length
  * @retval passed: pbuffer1 identical to pbuffer2
  *         failed: pbuffer1 differs from pbuffer2
  */
teststatus buffercmp(uint8_t* pbuffer1, uint8_t* pbuffer2, uint16_t bufferlength)
{
  while(bufferlength--)
  {
    if(*pbuffer1 != *pbuffer2)
    {
      return failed;
    }

    pbuffer1++;
    pbuffer2++;
  }

  return passed;
}

/**
  * @brief  initialize key and led
  * @param  none
  * @retval none
  */
void key_and_led_crl_init(void)
{
  gpio_init_type GPIO_InitStructure;
  crm_periph_clock_enable(CRM_GPIOA_PERIPH_CLOCK, TRUE);
  crm_periph_clock_enable(CRM_GPIOB_PERIPH_CLOCK, TRUE);
  crm_periph_clock_enable(CRM_GPIOE_PERIPH_CLOCK, TRUE);

  gpio_default_para_init(&GPIO_InitStructure);
  GPIO_InitStructure.gpio_pins = GPIO_PINS_0;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_INPUT;
  GPIO_InitStructure.gpio_pull = GPIO_PULL_DOWN;
  gpio_init(GPIOA, &GPIO_InitStructure);

  GPIO_InitStructure.gpio_pins = GPIO_PINS_9;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_OUTPUT;
  gpio_init(GPIOA, &GPIO_InitStructure);
  gpio_bits_reset(GPIOA, GPIO_PINS_9);

  GPIO_InitStructure.gpio_pins = GPIO_PINS_15;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_OUTPUT;
  gpio_init(GPIOE, &GPIO_InitStructure);
  gpio_bits_reset(GPIOE, GPIO_PINS_15);
}

/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  static u8 bartype = 0;
  uint8_t result[512] = {0} ;
  uint8_t result_compare[1024] = {0} ;
  uint8_t Play_times = 0 ;
  uint8_t data_len;
  
  system_clock_config();

  at32_board_init();

  nvic_configuration();

  uart_print_init(115200);	
  key_and_led_crl_init();
  exintx_init();

  /* configure sensor clock */
  clock_output_config(); 

  /* initialize spi for bf3901 */
  lcd_spi1_init(); 
  lcd_init();			
  lcd_display_initial_info();

  bf3901_spi_init(); 	

  bf3901_sccb_init();

  sccb_wr_reg(0x12, 0x80);

  while(bf3901_init())
  {
    lcd_showstring(5, 70, 200, 16, 16, "BF3901 Error!!");
    delay_ms(200);
  }

  lcd_showstring(5, 70, 200, 16, 16, "BF3901 Init OK");

  printf("1SRAM IN:%d\r\n", my_mem_perused(SRAMIN));

  /* initialize QR decode lib */
  Artery_QR_init();

  printf("2SRAM IN:%d\r\n", my_mem_perused(SRAMIN));

  lcd_showstring(5, 120, 200, 20, 16, "Please push the USER key to test QR decode!!");

  /* usb configure */
  /* select usb 48m clcok source */
  usb_clock48m_select(USB_CLK_HEXT);
  /* enable usb clock */
  crm_periph_clock_enable(CRM_USB_PERIPH_CLOCK, TRUE);
  /* enable usb interrupt */
  nvic_irq_enable(USBFS_L_CAN1_RX0_IRQn, 0, 0);
  /* usb core init */
  usbd_core_init(&usb_core_dev, USB, &cdc_class_handler, &cdc_desc_handler, 0);
  /* enable usb pull-up */
  usbd_connect(&usb_core_dev);

  /* waiting for key press */
  while(Key_run != 0x55);

  Key_run = 0x22;
  dma_spix_enable();
  delay_ms(500);
  lcd_clear(WHITE);
  blockwrite((lcddev.width - LCD_WIDTH) / 2, (lcddev.width + LCD_WIDTH) / 2 - 1, (lcddev.height - LCD_HEIGHT) / 2, (lcddev.height + LCD_HEIGHT) / 2 - 1);
	#if BF3901_BOARD_VERSION == 1
  exint14_init();
	#elif BF3901_BOARD_VERSION == 2
	exint12_init();
	#endif

  while(1)
  {
    camera_refresh();

    /* running ATK_QR */
    while(Get_Frame == 1)
    {
      /* decode qr code,  each time takes 0.2s */
      Artery_QR__decode(240, 240, bmp, bartype, result);

      /* unable to decode */
      if(result[0] == 0)
      {
#if defined(Only_detect_QR)
        printf("Not detect !  ");
        bartype = 0;
        Get_Frame = 0;
#elif defined(Detect_all)
        bartype++;

        if(bartype >= 5)
        {
          printf("Not detect !  ");
          bartype = 0;
          Get_Frame = 0;
        }

#endif

        if(Key_run == 0x55)
        {
          lcd_clear(WHITE);
          blockwrite((lcddev.width - LCD_WIDTH) / 2, (lcddev.width + LCD_WIDTH) / 2 - 1, (lcddev.height - LCD_HEIGHT) / 2, (lcddev.height + LCD_HEIGHT) / 2 - 1);
          delay_sec(2);
          Key_run = 0x22;
        }
      }
      else if(result[0] != 0)  /* decode success */
      {
        /* buzzer switch on */
        gpio_bits_set(GPIOE, GPIO_PINS_15);
        
        if(buffercmp(result_compare, result, sizeof(result)) == 1)
        {
          Play_times++;
        }
        else
        {
          Play_times = 1;
        }

        /* dispaly data to lcd */
        lcd_clear(WHITE);
        lcd_showstring(5, 10, 200, 16, 16, "The times:");
        lcd_shownum(130, 10, Play_times, 2, 16);
        lcd_showstring(5, 30, 200, 16, 16, "The result :");
        lcd_showstring(5, 50, 200, 100, 16, result);
        lcd_showstring(5, 190, 200, 20, 16, "Please push the USER key to test QR decode!!");
        printf("\r\nresult:\r\n%s\r\n", result);

        /* transmit data to usb */
        data_len = strlen((const char*)result);
        usb_vcp_send_data(&usb_core_dev, result, data_len);

        at32_led_toggle(LED3);
        memcpy(result_compare, result, sizeof(result));
        Key_run = 0x66;

        if(Key_run != 0x66)
        {
          delay_sec(1);
          lcd_clear(WHITE);
          blockwrite((lcddev.width - LCD_WIDTH) / 2, (lcddev.width + LCD_WIDTH) / 2 - 1, (lcddev.height - LCD_HEIGHT) / 2, (lcddev.height + LCD_HEIGHT) / 2 - 1);
        }

        Get_Frame = 0;
        gpio_bits_reset(GPIOE, GPIO_PINS_15);
      }
    }

    if(Key_run == 0x11)
    {
      Get_Frame = 0;
      Line_Refresh_Flg = RX_IDLE_FLG;
      delay_sec(1);
      lcd_clear(WHITE);
      blockwrite((lcddev.width - LCD_WIDTH) / 2, (lcddev.width + LCD_WIDTH) / 2 - 1, (lcddev.height - LCD_HEIGHT) / 2, (lcddev.height + LCD_HEIGHT) / 2 - 1);
      delay_sec(1);
      Key_run = 0x22;
    }
  }
}

/**
  * @brief  this function handles usb interrupt.
  * @param  none
  * @retval none
  */
void USBFS_L_CAN1_RX0_IRQHandler(void)
{
  usbd_irq_handler(&usb_core_dev);
}

/**
  * @brief  usb delay millisecond function.
  * @param  ms: number of millisecond delay
  * @retval none
  */
void usb_delay_ms(uint32_t ms)
{
  /* user can define self delay function */
  delay_ms(ms);
}

/**
  * @brief  usb delay microsecond function.
  * @param  us: number of microsecond delay
  * @retval none
  */
void usb_delay_us(uint32_t us)
{
  delay_us(us);
}
